
#include "ModbusMaster.h"
#include "Arduino_JSON.h"
#include "config.h"
#include "config_node.h"
#include "hw_modbus.h"





bool view_config_mode = false;    
uint8_t viewmode_otaa_period = 0;
config_node_t cfg;

void usr_btn_handler(void);
void startup_config(void);
void lorawan_begin(void);
char *create_view_data(void);

/*
* Setup function.
*/
void setup(void) {
    brd_hw_init(usr_btn_handler);

    startup_config();

    // lorawan_begin();

    mb_hw_init();
    mb_config_desc(cfg.mbdesc);
}

/*
* Loop program.
*/
void loop(void) {
    led_on();
    mb_reqdata();

    char *viewdata = create_view_data();
    mb_release_data();
    
    led_off();

    if(view_config_mode == true){
        
        Serial.println(viewdata);

        viewmode_otaa_period++;
        if(viewmode_otaa_period * DATAVIEW_PERIOD >=  OTAA_PERIOD){
    //         // lorawan_send((uint8_t *)viewdata, strlen(viewdata));
            Serial.println("LoRaWAN send in viewmode");
        }

        if(viewdata) free(viewdata);
        delay(DATAVIEW_PERIOD);
    }
    else{  
        Serial.println(viewdata);
        
        // lorawan_send((uint8_t *)viewdata, strlen(viewdata));
        Serial.println("LoRaWAN send in normal");

        if(viewdata) free(viewdata);
        api.system.sleep.all(OTAA_PERIOD);
        api.system.sleep.all(OTAA_PERIOD);
    }
}

void usr_btn_handler(void){
    delay(50);

    if(digitalRead(USR_BTN_PIN) == LOW){
        delay(100);

        view_config_mode = !view_config_mode;
        Serial.println("Switch mode");
        (view_config_mode == true)? wf_pwron() : wf_pwroff();
    }
}


void startup_config(void){
    led_on();

    wf_pwron();
    bool config_success = config_node(&cfg);
    wf_pwroff();
    if(config_success == false) api.system.reboot();

    led_off();

    delay(100);
}

void lorawan_begin(void){
    led_on();

    lorawan_init(&cfg.lorawan);
    lorawan_join();
    lorawan_start();

    led_off();
}

char *create_view_data(void){
    char *s;
    JSONVar root;
    JSONVar mbdata;

    for(uint8_t i=0; i<mb_set_size(); i++){
        if(mb_select_slave(i)->res == 1)
            mbdata[(const char *)mb_select_slave(i)->desc] = mb_select_slave(i)->rdata[0];
    }

    root["batt"] = (int)(batt_voltage() * 100.0);
    root["mbdata"] = mbdata;

    char *json_str = (char *)JSON.stringify(root).c_str();
    uint16_t len = strlen(json_str);
    s = (char *)malloc(len * sizeof(char));
    memcpy(s, json_str, len);

    return s;
}















